import pika


class BaseQueueManager:

    def __init__(self, amqp_url, exchange_name):
        self.credentials = pika.URLParameters(amqp_url)
        self.exchange_name = exchange_name

    def __enter__(self):
        self.connection = pika.BlockingConnection(self.credentials)
        self.channel = self.connection.channel()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.connection.close()



from queue_manager.base_queue_manager import BaseQueueManager


class QueueConsumer(BaseQueueManager):

    def __enter__(self):
        super(QueueConsumer, self).__enter__()
        self._bind_to_exchange()
        return self

    def _bind_to_exchange(self) -> None:
        result = self.channel.queue_declare(queue='', exclusive=True)
        self.channel.queue_bind(exchange=self.exchange_name, queue=result.method.queue)
        self._tmp_queue_name = result.method.queue

    def start_consuming(self, message_callback):
        self.channel.basic_consume(queue=self._tmp_queue_name, on_message_callback=message_callback, exclusive=True)
        self.channel.start_consuming()



from pika import BasicProperties
from queue_manager.base_queue_manager import BaseQueueManager


class QueueProducer(BaseQueueManager):

    def __init__(self, queue_arguments, *args, **kwargs):
        super(QueueProducer, self).__init__(*args, **kwargs)
        self.queue_arguments = queue_arguments

    def publish(self, priority, message):
        self.channel.queue_declare(queue='', arguments=self.queue_arguments, exclusive=True)
        properties = BasicProperties(priority=priority)
        self.channel.basic_publish(exchange=self.exchange_name, routing_key='', body=message, properties=properties)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Queue Manager",
    version="0.0.1",
    author="Barad & Riftin & Ori",
    description="A library for using queues as both a producer and a consumer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/riftool/queue-manager",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
